
/******************************************************************************
* MODULE     : averages.hpp
* DESCRIPTION: Moulian calculus with averages
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_AVERAGES_HPP
#define __MMX_AVERAGES_HPP
#include "mould.hpp"
namespace mmx {
#define TMPL template<typename C, typename X, typename V>
#define Mould mould<C,X,V>
#define Mould_rep mould_rep<C,X,V>
#define M list<X>

/******************************************************************************
* Lateral moulds
******************************************************************************/

TMPL Mould
l_mould () {
  return Mould (C(1), M (X ("-")));
}

TMPL Mould
r_mould () {
  return Mould (C(1), M (X ("+")));
}

TMPL Mould
mul_mould () {
  return invert (1 - l_mould<C,X,V> ());
}

TMPL Mould
mur_mould () {
  return invert (1 - r_mould<C,X,V> ());
}

TMPL Mould
rul_mould () {
  return mul_mould<C,X,V> () / mur_mould<C,X,V> ();
}

TMPL Mould
lur_mould () {
  return mur_mould<C,X,V> () / mul_mould<C,X,V> ();
}

TMPL Mould
dun_mould () {
  return log (lur_mould<C,X,V> ());
}

TMPL Mould
mun_mould () {
  return exp (dun_mould<C,X,V> () / 2) * mul_mould<C,X,V> ();
}

/******************************************************************************
* Average moulds
******************************************************************************/

template<typename X> table<bool,X>
lr_table () {
  table<bool,X> t (false);
  t (X ("+"))= true;
  t (X ("-"))= true;
  return t;
}

TMPL
class mon_mould_rep: public Mould_rep {
public:
  mon_mould_rep () {}
  generic expression () const { return "mon"; }
  table<bool,X> compute_deps () { return lr_table<X> (); }
  C compute (const M& x) {
    if (N(x) == 0) return C (1);
    C r= C(1) / C(2);
    for (int i=1; i<((int) N(x)); i++)
      if (x[i] != x[i-1]) r= r / C(2*i+2);
      else r= r * (C(1) - C(1) / C(2*i+2));
    return r;
  }
};

TMPL Mould
mon_mould () {
  return (Mould_rep*) new mon_mould_rep<C,X,V> ();
}

TMPL
class dom_mould_rep: public Mould_rep {
public:
  dom_mould_rep () {}
  generic expression () const { return "dom"; }
  table<bool,X> compute_deps () { return lr_table<X> (); }
  C compute (const M& x) {
    if (N(x) == 0) return C (0);
    int i, ch= 0;
    for (i=0; i<((int) N(x))-2; i++)
      if (x[i] != x[i+1]) ch++;
    C eps (x[N(x)-1] == X ("+")? 1: -1);
    if (ch == 0) return eps / C (N(x));
    if (ch == 1) return eps / C (2*N(x));
    return C(0);
  }
};

TMPL Mould
dom_mould () {
  return (Mould_rep*) new dom_mould_rep<C,X,V> ();
}

#undef TMPL
#undef Mould
#undef Mould_rep
#undef M
} // namespace mmx
#endif // __MMX_AVERAGES_HPP
