
/******************************************************************************
* MODULE     : mv_series_test.cc
* DESCRIPTION: Test multivariate series
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "mv_series.hpp"
#include "linear_series.hpp"
using namespace mmx;

#define Series series<rational,multivariate<naive_multiply> >

int
main () {
  Series x (generic ("x"));
  Series y (generic ("y"));
  Series z (generic ("z"));
  Series f= exp (x + y * exp (z));

  //cout << "x\t\t= " << x << "\n";
  //cout << "y\t\t= " << y << "\n";
  //cout << "x+y\t\t= " << x+y << "\n";
  //cout << "exp(x+y)\t= " << exp(x+y) << "\n";
  //cout << "x+y+exp(x+y)\t= " << (x+y)+exp(x+y) << "\n";

  cout << "x\t\t= " << x << " + ...\n";
  cout << "y\t\t= " << y << " + ...\n";
  cout << "exp(x)\t\t= " << exp (x) << " + ...\n";
  cout << "x+y\t\t= " << x + y << " + ...\n";
  cout << "log(1+x+y)\t= " << log (1 + x + y) << " + ...\n";
  cout << "exp(x+y)\t= " << exp (x + y) << " + ...\n";
  cout << "cos(x+y)\t= " << cos (x + y) << " + ...\n";
  cout << "sin(x+y)\t= " << sin (x + y) << " + ...\n";
  cout << "tan(x+y)\t= " << tan (x + y) << " + ...\n";
  cout << "exp(x+sin(y+z))\t= " << exp (x + sin (y + z)) << " + ...\n\n";

  cout << "f\t\t= " << f << " + ...\n";
  cout << "df/dx\t\t= " << derive (f, "x") << " + ...\n";
  cout << "df/dy\t\t= " << derive (f, "y") << " + ...\n";
  cout << "df/dz\t\t= " << derive (f, "z") << " + ...\n";
  cout << "I(f,x)\t\t= " << integrate (f, "x") << " + ...\n";
  cout << "I(f,y)\t\t= " << integrate (f, "y") << " + ...\n";
  cout << "I(f,z)\t\t= " << integrate (f, "z") << " + ...\n";

  return 0;
}
