
/******************************************************************************
* MODULE     : real_ball_test.cc
* DESCRIPTION: Test real balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real_ball.hpp"
//#include "floating.hpp"
//#include "interval.hpp"
using namespace mmx;

#define Ball real_ball
//#define Ball interval<floating<> >

void
test_constants () {
  cout << "*** Remarkable constants\n\n";
  //cout << "epsilon\t= " << Epsilon (Ball) << "\n";
  cout << "fuzzy\t= " << Fuzz (Ball) << "\n";
  cout << "\n";
}

void
test_add_simple (int prec) {
  set_precision (prec);
  cout << "*** Additive arithmetic at precision " << prec << "\n\n";
  Ball x= 2;
  Ball y= 7;
  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "-x\t= " << (-x) << "\n";
  cout << "x+x\t= " << (x+x) << "\n";
  cout << "x+y\t= " << (x+y) << "\n";
  cout << "x-x\t= " << (x-x) << "\n";
  cout << "x-y\t= " << (x-y) << "\n";
  cout << "(x-x)+y\t= " << ((x-x)+y) << "\n";
  cout << "\n";
}

void
test_mul_simple (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative arithmeric at precision " << prec << "\n\n";
  Ball x= Ball (2) + Ball (1);
  cout << "x\t= " << x << "\n";
  cout << "x * x\t= " << x*x << "\n";
  cout << "x * -x\t= " << x*(-x) << "\n";
  cout << "-x * x\t= " << (-x)*x << "\n";
  cout << "-x * -x\t= " << (-x)*(-x) << "\n";
  cout << "x / x\t= " << (x/x) << "\n";
  cout << "x / -x\t= " << (x/(-x)) << "\n";
  cout << "-x / x\t= " << ((-x)/x) << "\n";
  cout << "-x / -x\t= " << ((-x)/(-x)) << "\n";
  cout << "x*x / x\t= " << ((x*x)/x) << "\n";
  cout << "\n";
}

void
test_other_ops (int prec) {
  set_precision (prec);
  cout << "*** Other operations " << prec << "\n\n";
  Ball x= Ball (2);
  Ball y= make_interval<Ball> (real_ball_center (-3), real_ball_center (2));
  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "x << 5\t= " << (x << 5) << "\n";
  cout << "x >> 3\t= " << (x >> 3) << "\n";
  cout << "y << 5\t= " << (y << 5) << "\n";
  cout << "abs(x)\t= " << (abs (x)) << "\n";
  cout << "abs(-x)\t= " << (abs (-x)) << "\n";
  cout << "abs(y)\t= " << (abs (y)) << "\n";
  cout << "\n";
}

void
test_mul_progression (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative progression at precision " << prec << "\n\n";
  int i;
  Ball x= 3;
  Ball y= x;
  for (i=0; i<50; i++) {
    cout << i << "\t" << y << "\n";
    y *= x;
  }
  cout << "\n";
}

void
test_mul_overflow (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative overflows at precision " << prec << "\n\n";
  int i;
  Ball x= 3;
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x *= x;
  }
  cout << "\n";
}

void
test_mul_underflow (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative underflows at precision " << prec << "\n\n";
  int i;
  Ball x= 0.3;
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x *= x;
  }
  cout << "\n";
}

void
test_muller (int prec) {
  set_precision (prec);
  cout << "*** Muller's sequence at precision " << prec << "\n\n";
  int i;
  Ball a= 99;
  Ball b= -600;
  Ball d= -7;
  Ball x= 4;
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x= (a*x + b) / (x + d);
  }
  cout << "\n";
}

void
test_elementary_funcs (int prec) {
  set_precision (prec);
  cout << "*** Elementary functions at precision " << prec << "\n\n";
  Ball x= 1;
  Ball y= 3e20;
  Ball z= x - x/y;
  cout << "log (2)\t\t= " << Log2 (Ball) << "\n";
  cout << "pi\t\t= " << Pi (Ball) << "\n";
  cout << "euler\t\t= " << Euler (Ball) << "\n";
  cout << "NaN\t\t= " << Nan (Ball) << "\n";
  cout << "Infty\t\t= " << Plus_infinity (Ball) << "\n";
  cout << "Fuzz\t\t= " << Fuzz (Ball) << "\n";
  cout << "x\t\t= " << x << "\n";
  cout << "y\t\t= " << y << "\n";
  cout << "z\t\t= " << z << "\n";
  cout << "sqrt (x)\t= " << sqrt (x) << "\n";
  cout << "sqrt (y)\t= " << sqrt (y) << "\n";
  cout << "sqrt (-y)\t= " << sqrt (-y) << "\n";
  cout << "exp (x)\t\t= " << exp (x) << "\n";
  cout << "exp (y)\t\t= " << exp (y) << "\n";
  cout << "exp (-x)\t= " << exp (-x) << "\n";
  cout << "exp (-y)\t= " << exp (-y) << "\n";
  cout << "log (x)\t\t= " << log (x) << "\n";
  cout << "log (y)\t\t= " << log (y) << "\n";
  cout << "cos (x)\t\t= " << cos (x) << "\n";
  cout << "cos (y)\t\t= " << cos (y) << "\n";
  cout << "sin (x)\t\t= " << sin (x) << "\n";
  cout << "sin (y)\t\t= " << sin (y) << "\n";
  cout << "tan (x)\t\t= " << tan (x) << "\n";
  cout << "tan (y)\t\t= " << tan (y) << "\n";
  cout << "arc sin (1/2)\t= " << asin (x>>1) << "\n";
  cout << "arc sin (z)\t= " << asin (z) << "\n";
  cout << "arc cos (1/2)\t= " << acos (x>>1) << "\n";
  cout << "arc cos (z)\t= " << acos (z) << "\n";
  cout << "arc tan (x)\t= " << atan (x) << "\n";
  cout << "arc tan (y)\t= " << atan (y) << "\n";
  cout << "arc tan (x)\t= " << atan (x) << "\n";
  cout << "atan2 (x, y)\t= " << atan2 (x, y) << "\n";
  cout << "atan2 (y, x)\t= " << atan2 (y, x) << "\n";
  cout << "atan2 (0, -x)\t= " << atan2 (Ball (0), -x) << "\n";
  cout << "min (x, y)\t= " << min (x, y) << "\n";
  cout << "max (x, y)\t= " << max (x, y) << "\n";
  cout << "\n";
}

void
test_all (int prec) {
  test_add_simple (prec);
  test_mul_simple (prec);
  test_other_ops (prec);
  test_mul_progression (prec);
  test_mul_overflow (prec);
  test_mul_underflow (prec);
  test_muller (prec);
  test_elementary_funcs (prec);
}

int
main () {
  test_constants ();
  test_all (30);
  test_all (100);
  test_all (200);

  cout << "infty\t\t= " << Plus_infinity (Ball) << "\n";
  cout << "-infty\t\t= " << Minus_infinity (Ball) << "\n";
  cout << "infty+1\t\t= " << Plus_infinity (Ball) + Ball (1) << "\n";
  cout << "1-infty\t\t= " << Ball (1) - Plus_infinity (Ball) << "\n";
  cout << "1+(-infty)\t= " << Ball (1) + Minus_infinity (Ball) << "\n";
  cout << "2 infty\t\t= " << Ball (2) * Plus_infinity (Ball) << "\n";
  cout << "1/infty\t\t= " << Ball (1) / Plus_infinity (Ball) << "\n";
  cout << "infty^2\t\t= " <<
    Plus_infinity (Ball) * Plus_infinity (Ball) << "\n";
  cout << "(-infty) infty\t= " <<
    Minus_infinity (Ball) * Plus_infinity (Ball) << "\n";
  cout << "(-infty)^2\t= " <<
    Minus_infinity (Ball) * Minus_infinity (Ball) << "\n";
  cout << "infty/infty\t= " <<
    Plus_infinity (Ball) / Plus_infinity (Ball) << "\n";
  cout << "(-infty)/infty\t= " <<
    Minus_infinity (Ball) / Plus_infinity (Ball) << "\n";
  cout << "(-infty)/2\t= " << Minus_infinity (Ball) / Ball (2) << "\n";
  cout << "|infty|\t\t= " << abs (Plus_infinity (Ball)) << "\n";
  cout << "|-infty|\t= " << abs (Minus_infinity (Ball)) << "\n";
  cout << "center(infty)\t= " << center (Plus_infinity (Ball)) << "\n";
  cout << "center(-infty)\t= " << center (Minus_infinity (Ball)) << "\n";
  cout << "radius(infty)\t= " << radius (Plus_infinity (Ball)) << "\n";
  cout << "radius(-infty)\t= " << radius (Minus_infinity (Ball)) << "\n";
  cout << "infty << 3\t= " << (Plus_infinity (Ball) << 3) << "\n";
  cout << "infty >> 3\t= " << (Plus_infinity (Ball) >> 3) << "\n";
  cout << "(-infty) << 3\t= " << (Minus_infinity (Ball) << 3) << "\n";
  cout << "(-infty) >> 3\t= " << (Minus_infinity (Ball) >> 3) << "\n";
  return 0;
}
