
/******************************************************************************
* MODULE     : evaluator.hpp
* DESCRIPTION: Abstract evaluators for symbolic expressions
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __EVALUATOR_HPP
#define __EVALUATOR_HPP
#include "basix.hpp"
namespace mmx {
class evaluator_rep;
class evaluator;
class environment;

#define PENALTY_NONE             ((nat) 0)
#define PENALTY_AUTOMATIC        ((nat) 1)
#define PENALTY_INCLUSION        ((nat) 2)
#define PENALTY_HOMOMORPHISM     ((nat) 3)
#define PENALTY_PROMOTE_GENERIC  ((nat) 4)
#define PENALTY_CAST             ((nat) 5)
#define PENALTY_INVALID          ((nat) 6)

class evaluator_rep: public rep_struct {
public:
  virtual environment get_environment () const;
  virtual void set (const generic& var, const generic& val) const;
  virtual void reset (const generic& var) const;
  virtual void overload (const generic& var, const generic& val,
			 nat penalty= PENALTY_INCLUSION) const;
  virtual bool contains (const generic& var) const;
  virtual generic get (const generic& var) const;
  virtual bool read (const generic& var, generic& val) const;

  virtual generic eval  (const generic& x) const;
  virtual generic construct (const generic& x) const;
  virtual generic xapply (const generic& x) const;
  virtual generic apply (const generic& fun) const;
  virtual generic apply (const generic& fun, const generic& x1) const;
  virtual generic apply (const generic& fun,
			 const generic& x1, const generic& x2) const;

public:
  inline evaluator_rep () {}
  inline virtual ~evaluator_rep () {}
  friend class evaluator;
};

class evaluator {
INDIRECT_PROTO (evaluator, evaluator_rep)
public:
  inline evaluator (): rep (new evaluator_rep ()) {}
  friend class evaluator_rep;
};
INDIRECT_IMPL (evaluator, evaluator_rep)

extern evaluator current_ev;

void select_evaluator (const evaluator& ev);
void select_output_evaluator ();
void restore_evaluator ();

} // namespace mmx
#endif // __EVALUATOR_HPP
