
/******************************************************************************
* MODULE     : compound.cpp
* DESCRIPTION: Compound generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "compound.hpp"
#include "generic_internal.hpp"
namespace mmx {

array<generic>
compound_to_array (const generic& g) {
  return *as<compound> (g);
}

generic
array_to_compound (const array<generic>& v) {
  return as<generic> (compound (v));
}

generic
flatten (const compound& c) {
  array<generic> v= *c;
  nat i, n= N(v);
  array<generic> w= array_fill<generic> (n);
  for (i=0; i<n; i++)
    w(i)= flatten (v[i]);
  return array_to_compound (w);
}

generic
generic::operator () () const {
  return new generic_concrete_rep<compound> (array<generic> (*this));
}

generic
generic::operator () (const generic& g1) const {
  return new generic_concrete_rep<compound> (array<generic> (*this, g1));
}

generic
generic::operator () (const generic& g1, const generic& g2) const {
  return new generic_concrete_rep<compound> (array<generic> (*this, g1, g2));
}

generic
generic::operator () (const generic& g1, const generic& g2,
		      const generic& g3) const {
  return new generic_concrete_rep<compound>
               (array<generic> (*this, g1, g2, g3));
}

generic
gen (const generic& x1, const array<generic>& v) {
  return as<generic> (compound (cons (x1, v)));
}

generic
ugen (const generic& x1, const array<generic>& v) {
  return new generic_concrete_rep<compound> (cons (x1, v), true);
}

generic
gen () {
  return as<generic> (compound (array<generic> ()));
}

generic
ugen () {
  return new generic_concrete_rep<compound> (array<generic> (), true);
}

generic
gen (const generic& x1) {
  return as<generic> (compound (array<generic> (x1)));
}

generic
ugen (const generic& x1) {
  return new generic_concrete_rep<compound> (array<generic> (x1), true);
}

generic
gen (const generic& x1, const generic& x2) {
  return as<generic> (compound (array<generic> (x1, x2)));
}

generic
ugen (const generic& x1, const generic& x2) {
  return new generic_concrete_rep<compound> (array<generic> (x1, x2), true);
}

generic
gen (const generic& x1, const string& x2) {
  return as<generic> (compound (array<generic> (x1, generic (x2))));
}

generic
ugen (const generic& x1, const string& x2) {
  return new generic_concrete_rep<compound>
               (array<generic> (x1, generic(x2)), true);
}

generic
gen (const generic& x1, const generic& x2, const generic& x3) {
  return as<generic> (compound (array<generic> (x1, x2, x3)));
}

generic
ugen (const generic& x1, const generic& x2, const generic& x3) {
  return new generic_concrete_rep<compound>
               (array<generic> (x1, x2, x3), true);
}

generic
gen (const generic& x1, const generic& x2,
     const generic& x3, const generic& x4) {
  return as<generic> (compound (array<generic> (x1, x2, x3, x4)));
}

generic
ugen (const generic& x1, const generic& x2,
      const generic& x3, const generic& x4) {
  return new generic_concrete_rep<compound>
               (array<generic> (x1, x2, x3, x4), true);
}

generic
gen (const generic& x1, const generic& x2,
     const generic& x3, const generic& x4, const generic& x5) {
  return as<generic> (compound (array<generic> (x1, x2, x3, x4, x5)));
}

generic
ugen (const generic& x1, const generic& x2,
      const generic& x3, const generic& x4, const generic& x5) {
  return new generic_concrete_rep<compound>
               (array<generic> (x1, x2, x3, x4, x5), true);
}

generic
gen (const generic& x1, const generic& x2,
     const generic& x3, const generic& x4,
     const generic& x5, const generic& x6) {
  return as<generic> (compound (array<generic> (x1, x2, x3, x4, x5, x6)));
}

generic
ugen (const generic& x1, const generic& x2,
      const generic& x3, const generic& x4,
      const generic& x5, const generic& x6) {
  return new generic_concrete_rep<compound>
               (array<generic> (x1, x2, x3, x4, x5, x6), true);
}

} // namespace mmx
