<TeXmacs|1.0.6.9>

<style|<tuple|mmxdoc|mathemagix>>

<\body>
  <tmdoc-title|Coding conventions and packaging>

  <section|Introduction>

  Two types of contributions can be made to <name|Mathemagix>: purely
  external packages and packages which should be included and maintained into
  the main <name|Mathemagix> source tree. Purely external packages can
  typically be distributed along with a software library or scripting
  language. If you want to include your package inside the main
  <name|Mathemagix> source tree, then you should scrupulously follow a
  certain number of conventions.

  <section|Conventions for the C++ source code>

  The following conventions are used throughout the C++ source code:

  <\itemize>
    <item>Put all your code inside the <verbatim|mmx> namespace. New
    namespaces can be nested inside the <verbatim|mmx> namespace. Only
    exception: the principal glue definition routine
    <verbatim|define_<em|<with|color|brown|name>>> for a package
    <verbatim|<em|<with|color|brown|name>>> should be put unmangled in the
    global namespace.

    <item>Use the following naming and case conventions:

    <\itemize>
      <item>Lowercase for types (<verbatim|string>, <verbatim|vector>, etc.).

      <item>Short names with first letter in uppercase for template arguments
      (<verbatim|C>, <verbatim|V>, <verbatim|><verbatim|Op>, etc.).

      <item>Short lowercase names for variables. When possible, use the first
      letter of the type when possible, <abbr|i.e.> <verbatim|string s>,
      <verbatim|int i1>, etc.

      <item>Fully uppercase names for macros (<verbatim|INDIRECT_PROTO>,
      <verbatim|ARITH_INT_SUGAR>, etc.).

      <item>Exception to the above rule when the macro is an abbreviation for
      a type or a value, in which case only the first letter uses uppercase
      (<verbatim|Vector>, <verbatim|Matrix>, <verbatim|Pi>, etc.).
    </itemize>

    <item>Don't use STL, but rather build upon the common datatypes provided
    by <verbatim|basix> (<abbr|i.e.> <verbatim|string>, <verbatim|array>,
    <verbatim|list>, <verbatim|table>, <verbatim|function>,
    <verbatim|iterator>, <verbatim|symbol>, <verbatim|pair>,
    <verbatim|triple>). Also notice that <verbatim|nat> stands for
    <verbatim|unsigned int>.

    <item>When defining a data type <verbatim|<verbatim|my_type>> which is
    really a pointer to the representation class <verbatim|my_type_rep>, use
    the standard mechanism for defining reference counted pointer classes. In
    the case of a simple class <verbatim|my_type>, this is done using the
    following kind of code:

    <\cpp-fragment>
      class string {

      INDIRECT_PROTO (string, string_rep)

      \ \ ...

      };

      INDIRECT_IMPL (string, string_rep)
    </cpp-fragment>

    In the case of template classes, one should use similar macros
    <verbatim|INDIRECT_PROTO_1>, etc.

    <item>To increase conciseness, we systematically abbreviate template
    definitions and template type names when defining new template classes.
    For instance, the code of the <verbatim|list\<less\>C\<gtr\>> class is
    enclosed in a block of the form:

    <\cpp-fragment>
      namespace mmx {

      #define TMPL_DEF template\<less\>typename C\<gtr\>

      #define TMPL template\<less\>typename C\<gtr\>

      #define List list\<less\>C\<gtr\>

      #define List_rep list_rep\<less\>C\<gtr\>

      ...

      #undef TMPL_DEF

      #undef TMPL

      #undef List

      #undef List_rep

      } // namespace mmx
    </cpp-fragment>

    <item>Please let every file start and finish by one blank line. Please
    don't indent code inside namespace directives which span over a whole
    file.
  </itemize>

  There are probably more convention; please follow the style of existing
  code when writing new packages.

  <section|Convention for packaging>

  Writing packages which compile on all platforms and can easily be
  maintained is a complex task. The <name|Mathemagix> philosophy is to be as
  standard and simple as possible, by using the following guidelines:

  <\enumerate>
    <item>Systematically use <name|Automake>/<name|Autoconf>, even though
    these tools are very bureaucratic.

    <item>Keep <verbatim|configure.ac> and the <verbatim|Makefile.am> as
    striaghtforward as possible.

    <item>Use a uniform organization for the directory structure.
  </enumerate>

  More precisely, the directory structure of a new package <verbatim|foox>
  should be as follows:

  <\shell-fragment>
    foox

    \| configure.ac

    \| Makefile.am

    \| macros

    \| \| ac_prefix_config_h.m4

    \| \| mmx_module.m4

    \| \| <with|color|brown|<em|dependency>>.m4

    \| src

    \| \| foox

    \| \| \| include

    \| \| \| glue

    \| \| \| \| foox_glue.cpp

    \| \| \| script

    \| \| \| \| foox-config.in

    \| \| <with|color|brown|<em|subdir>>

    \| \| \| bench

    \| \| \| glue

    \| \| \| include

    \| \| \| src

    \| \| \| test

    \| \| target

    \| \| \| Makefile.am
  </shell-fragment>

  Of course, there may be multiple <verbatim|<with|color|brown|<em|dependency>>>:
  any external dependency of your package should come with a separate
  <verbatim|.m4> file. Similarly, you also have one
  <verbatim|<with|color|brown|<em|subdir>>> for each type of functionality of
  your package. A configuration file <verbatim|foox/src/foox/include/foox-config.h.in>
  should be created automatically using <verbatim|autoheader> and
  <verbatim|AC_PREFIX_CONFIG_H>. The file <verbatim|foox/Makefile.am> is very
  short and mainly refers to the global makefile
  <verbatim|foox/src/target/Makefile.am>.

  <subsection|The file <verbatim|configure.ac>>

  The file <verbatim|configure.ac> should be modeled on the corresponding
  file of <verbatim|basix>. In particular, you have to start with something
  like

  <\shell-fragment>
    AC_INIT([foox], [0.2.2], [bugs@mathemagix.org])

    AM_INIT_AUTOMAKE([-Wall -Werror foreign])

    AC_CONFIG_HEADER(src/foox/include/config.hpp)

    \;

    AC_PROG_CC

    AC_PROG_CXX

    AC_LANG_CPLUSPLUS

    AC_PROG_INSTALL

    AC_PROG_LIBTOOL

    AC_PROG_RANLIB
  </shell-fragment>

  and end with something like

  <\shell-fragment>
    AC_OUTPUT([

    \ \ Makefile

    \ \ src/target/Makefile

    \ \ src/foox/script/foox-config

    ],[chmod +x src/foox/script/foox-config])

    \;

    AC_PREFIX_CONFIG_H(FOOX,[

    \ \ src/foox/include/config.hpp

    ],[

    \ \ src/foox/include/foox-config.hpp

    ])
  </shell-fragment>

  The file <verbatim|mmx_module.m4> defines a convenient macro
  <verbatim|AC_MMX_MODULE>. When calling this macro inside your
  <verbatim|configure.ac>, the following common options for <name|Mathemagix>
  packages will be supported.

  <\description>
    <item*|<verbatim|--enable-debug>>Enable debugging (disabled by default).

    <item*|<verbatim|--enable-optimize>>Enable optimization (enabled by
    default).

    <item*|<verbatim|--enable-test>>Build suite of test programs (disabled by
    default).

    <item*|<verbatim|--enable-bench>>Build suite of benchmark programs
    (disabled by default).
  </description>

  <\remark>
    <with|color|red|To do: options <verbatim|--enable-glue>,
    <verbatim|--enable-exceptions>, <verbatim|--enable-safeguards>.>
  </remark>

  Additional dependencies should all come with a corresponding <verbatim|m4>
  file. For instance, <verbatim|gnumerix> tests for <verbatim|gmp> in the
  following way:

  <\shell-fragment>
    AC_WITH_GMP

    AC_LIB_GMP
  </shell-fragment>

  The macro <verbatim|AC_WITH_GMP> provides an option to specify the install
  path for <verbatim|gmp>. The macro <verbatim|AC_LIB_GMP> tests whether it
  is possible to execute some functionality of the library. Notice that
  the<nbsp><verbatim|m4> files of some of the <name|Mathemagig> packages
  provide similar macros <verbatim|AC_WITH_BASIX>, <verbatim|AC_LIB_BASIX>,
  etc.

  <subsection|The file <verbatim|Makefile.am>>

  Each package should come with a global <verbatim|Makefile.am> which should
  be kept as straightforward as possible. First of all, all include files in
  <verbatim|foox/src/*/include/> and <verbatim|foox/src/*/glue/> should be
  put into the <verbatim|mathemagix> subdirectory of <verbatim|includedir>:

  <\shell-fragment>
    mmxincludedir = $(includedir)/mathemagix

    \;

    mmxinclude_HEADERS = \\

    \ \ ../<em|<with|color|brown|subdir1>>/include/<em|<with|color|brown|file1>>.hpp
    \\

    \ \ ../<em|<with|color|brown|subdir2>>/include/<em|<with|color|brown|file2>>.hpp
    \\

    \ \ ../<em|<with|color|brown|subdir1>>/glue/<em|<with|color|brown|file3>>.hpp
    \\

    \ \ ...
  </shell-fragment>

  One next has to specify the source files for the main <name|Libtool>
  library <verbatim|libfoox.la>:

  <\shell-fragment>
    libfoox_la_CPPFLAGS = \\

    \ \ -I../<em|<with|color|brown|subdir1>>/include \\

    \ \ -I../<em|<with|color|brown|subdir2>>/include \\

    \ \ ... \\

    \ \ @CPPFLAGS@

    \;

    libfoox_la_SOURCES = \\

    \ \ ../<em|<with|color|brown|subdir1>>/src/<em|<with|color|brown|foo1>>.cpp
    \\

    \ \ ../<em|<with|color|brown|subdir1>>/src/<em|<with|color|brown|foo2>>.cpp
    \\

    \ \ ...
  </shell-fragment>

  The source file for the glue library <verbatim|libmmxfoox.la> is specified
  similarly:

  <\shell-fragment>
    libmmxfoox_la_CPPFLAGS = \\

    \ \ -I../<em|<with|color|brown|subdir1>>/glue \\

    \ \ -I../<em|<with|color|brown|subdir2>>/glue \\

    \ \ ... \\

    \ \ $(libbasix_la_CPPFLAGS)

    \;

    libmmxfoox_la_SOURCES = \\

    \ \ ../<em|<with|color|brown|subdir1>>/glue/<em|<with|color|brown|glue1>>.cpp
    \\

    \ \ ../<em|<with|color|brown|subdir1>>/glue/<em|<with|color|brown|glue2>>.cpp
    \\

    \ \ ...
  </shell-fragment>

  and we set

  <\shell-fragment>
    lib_LTLIBRARIES = libfoox.la libmmxfoox.la
  </shell-fragment>

  We also have to provide a configuration script
  <verbatim|foox/src/foox/script/foox-config.in> on the same model as
  <verbatim|basix-config.in> and distribute it with our package:

  <\shell-fragment>
    dist_bin_SCRIPTS = ../basix/script/basix-config

    EXTRA_DIST = ../basix/script/basix-config.in
  </shell-fragment>

  In addition, one may wish to include some binaries in the case when one of
  the options <verbatim|--enable-test> or <verbatim|--enable-bench> was
  given. Please mimick the code in <verbatim|algebramix/target/Makefile.am>.

  <subsection|Inclusion in the global <name|Mathemagix> project>

  When your package is finished, according to the above guidelines, then we
  can inlcude it in the main <name|Mathemagix> tree. This involves some
  additional copy and pasting in the files <verbatim|configure.in>,
  <verbatim|Makefile.in> and <verbatim|set-devel-paths>. Although we will
  take care of that at the end, you may wish to make the necessary
  modifications yourself in order to compile all your packages at once.
</body>