AC_DEFUN([AC_MMX_OPTION_DEBUG], [
    if test "$CXXFLAGS" = "-g -O2"; then
        CXXFLAGS=""
    fi

    AC_ARG_ENABLE(debug,
    AC_HELP_STRING(
        [--enable-debug],
        [install a debugging enable executable [[-ggdb]]]),
    [], [enable_debug="no"])

    if test "$enable_debug" = "yes"; then
        if test "$GXX" = "yes"; then
            enable_debug="-ggdb"
        else
            enable_debug="-g3"
        fi
    fi

    AH_TEMPLATE([DEBUG_ON],[Compile with debugging information and checks])

    if test "$enable_debug" = "no"; then
        AC_MSG_RESULT([disabling debugging])
    else
        AC_MSG_RESULT([enabling debugging, $enable_debug])
        AC_DEFINE([DEBUG_ON],[])
        CXXFLAGS="$CXXFLAGS $enable_debug"
    fi
])

AC_DEFUN([AC_MMX_OPTION_OPTIMIZE], [
    if test "$CXXFLAGS" = "-g -O2"; then
        CXXFLAGS=""
    fi

    AC_ARG_ENABLE(optimize,
    AC_HELP_STRING(
        [--enable-optimize],
        [compile with optimizations [[guessed]]]),
    [], [enable_optimize="yes"])

    case "$enable_optimize" in
        yes)
	    CXXFLAGS="$CXXFLAGS -ansi -Wall -O3 -fexpensive-optimizations"
	    AC_MSG_RESULT([enabling optimizations, $CXXFLAGS]) ;;
	no)
	    AC_MSG_RESULT([using standard optimization flags, $CXXFLAGS]) ;;
	*)
	    CXXFLAGS="$CXXFLAGS $enable_optimize"
	    AC_MSG_RESULT([customizing optimizations, $CXXFLAGS]) ;;
    esac

    CXXFLAGS="$CXXFLAGS -ftemplate-depth-25"
])

AC_DEFUN([AC_MMX_OPTION_TEST], [
    AC_ARG_ENABLE(test,
    AC_HELP_STRING(
	[--enable-test],
	[compile additional test programs]),
    [], [enable_test="no"])

    AM_CONDITIONAL([TEST_OPT], [test "$enable_test" = yes])
])

AC_DEFUN([AC_MMX_OPTION_BENCH], [
    AC_ARG_ENABLE(bench,
    AC_HELP_STRING(
	[--enable-bench],
	[compile additional benchmark programs]),
    [], [enable_bench="no"])

    AM_CONDITIONAL([BENCH_OPT], [test "$enable_bench" = yes])
])

AC_DEFUN([AC_MMX_MODULE], [
    AC_MMX_OPTION_DEBUG
    AC_MMX_OPTION_OPTIMIZE
    AC_MMX_OPTION_TEST
    AC_MMX_OPTION_BENCH
])
