/******************************************************************************
* MODULE     : shell.hpp
* DESCRIPTION: Main routines for mmx-light
* COPYRIGHT  : (C) 2007 Gregoire Lecerf
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SHELL_HPP
#define __SHELL_HPP

#include <string.hpp>
#include <list.hpp>
#include <parse_tools.hpp>
#include <mmxlight-config.hpp>

namespace mmx {

// global variables that can be modified during the shell session
extern bool quiet_mode;
extern bool debug_mode;
extern bool type_mode;
extern bool time_mode;
extern bool math_mode;

// global variables set for the whole session
extern bool completion_mode;
extern bool texmacs_mode;

// identifier list for completion
extern list<string> identifiers_for_completion;
  
string get_source (nat input_number, nat line);
  // only used in interactive mode

nat input_number (void);
  // only used in interactive mode

list<string> shell_initialize (int argc, char** argv);
bool shell_terminal_input (string& in);
bool shell_terminal_batch_input (string& in);
void shell_terminal_output (const generic& g, const generic& t);
bool shell_terminal_print_syntax_errors (const list<syntax_error>& errors);
bool shell_terminal_print_exception (const generic& g);
void shell_texmacs_initialize (void);
bool shell_texmacs_input (string& in);
void shell_texmacs_output (const generic& g, const generic& t);

} // namespace mmx
#endif // __SHELL_HPP
