
/******************************************************************************
* MODULE     : symbolic.hpp
* DESCRIPTION: Symbolic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLIC_HPP
#define __SYMBOLIC_HPP
#include "generic_internal.hpp"
#include "symbolix_glue.hpp"
namespace mmx {
#define TMPL_DEF template<typename C,typename V=basic_symbolic<C> >
#define TMPL template<typename C,typename V>
#define Symbolic symbolic<C,V>
TMPL class symbolic_rep;
TMPL class symbolic;

/******************************************************************************
* Symbolic variants
******************************************************************************/

template<typename C>
struct basic_symbolic {
  static evaluator ev;
};

template<typename C> evaluator
basic_symbolic<C>::ev= symbolix_evaluator<C> ();

/******************************************************************************
* Symbolic class
******************************************************************************/

TMPL_DEF
class symbolic {
private:
  generic rep;

public:
  inline symbolic (const generic& g): rep (g) {}
  inline generic operator * () const { return rep; }
  inline symbolic (char* s): rep (s) {}
  inline symbolic (const string& s): rep (s) {}
  inline Symbolic operator [] (nat i) const { return rep[i]; }

  symbolic ();
  symbolic (int i);
  symbolic (const C& sc);
  Symbolic operator () () const;
  Symbolic operator () (const Symbolic& x1) const;
  Symbolic operator () (const Symbolic& x1, const Symbolic& x2) const;
  Symbolic operator () (const Symbolic& x1, const Symbolic& x2,
			const Symbolic& x3) const;
};

TMPL inline nat N (const Symbolic& x) { return N (*x); }
TMPL inline nat hash (const Symbolic& x) { return hash (*x); }

/******************************************************************************
* Lifting operators on generics
******************************************************************************/

template<typename C, typename V, typename X,
	 generic (*fun) (const X&)>
inline generic
lift (const X& x) {
  select_evaluator (V::ev);
  generic ret= fun (x);
  restore_evaluator ();
  return ret;
}

template<typename C, typename V,
	 generic (*fun) (const generic&)>
inline generic
lift (const Symbolic& x) {
  select_evaluator (V::ev);
  generic ret= fun (*x);
  restore_evaluator ();
  return ret;
}

template<typename C, typename V,
	 generic (*fun) (const generic&, const generic&)>
inline generic
lift (const Symbolic& x, const Symbolic& y) {
  select_evaluator (V::ev);
  generic ret= fun (*x, *y);
  restore_evaluator ();
  return ret;
}

template<typename C, typename V,
	 generic (*fun) (const generic&, const generic&, const generic&)>
inline generic
lift (const Symbolic& x, const Symbolic& y, const Symbolic& z) {
  select_evaluator (V::ev);
  generic ret= fun (*x, *y, *z);
  restore_evaluator ();
  return ret;
}

template<typename C, typename V,
	 generic (*fun) (const generic&, const generic&,
			 const generic&, const generic&)>
inline generic
lift (const Symbolic& x1, const Symbolic& x2,
      const Symbolic& x3, const Symbolic& x4) {
  select_evaluator (V::ev);
  generic ret= fun (*x1, *x2, *x3, *x4);
  restore_evaluator ();
  return ret;
}

template<typename C, typename V,
	 bool (*fun) (const generic&, const generic&)>
inline bool
lift (const Symbolic& x, const Symbolic& y) {
  select_evaluator (V::ev);
  bool ret= fun (*x, *y);
  restore_evaluator ();
  return ret;
}

/******************************************************************************
* Constructors and printing
******************************************************************************/

template<typename C, typename T> inline C
std_constructor (const T& x) {
  return C (x);
}

template<typename C, typename V, typename T> inline generic
constructor (const T& x) {
  return lift<C,V,T,std_constructor<generic,T> > (x);
}

TMPL Symbolic::symbolic ():
  rep (constructor<C,V,C> (C(0))) {}
TMPL Symbolic::symbolic (int i):
  rep (constructor<C,V,C> (C(i))) {}
TMPL Symbolic::symbolic (const C& sc):
  rep (constructor<C,V,C> (sc)) {}
TMPL Symbolic Symbolic::operator () () const {
  return lift<C,V,apply> (*this); }
TMPL Symbolic Symbolic::operator () (const Symbolic& x1) const {
  return lift<C,V,apply> (*this, x1); }
TMPL Symbolic Symbolic::operator ()
  (const Symbolic& x1, const Symbolic& x2) const {
    return lift<C,V,apply> (*this, x1, x2); }
TMPL Symbolic Symbolic::operator ()
  (const Symbolic& x1, const Symbolic& x2, const Symbolic& x3) const {
    return lift<C,V,apply> (*this, x1, x2, x3); }

TMPL generic flatten (const Symbolic& x) {
  return flatten (*x); }

PRINT_SUGAR(TMPL,Symbolic)

/******************************************************************************
* Operators
******************************************************************************/

TMPL Symbolic operator - (const Symbolic& x1) {
  return lift<C,V,neg_operator> (x1); }
TMPL Symbolic operator + (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,add_operator> (x1, x2); }
TMPL Symbolic operator - (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,sub_operator> (x1, x2); }
TMPL Symbolic operator * (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,mul_operator> (x1, x2); }
TMPL Symbolic operator / (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,div_operator> (x1, x2); }
TMPL Symbolic quo (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,quo> (x1, x2); }
TMPL Symbolic rem (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,rem> (x1, x2); }
TMPL Symbolic pow (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,pow> (x1, x2); }
TMPL Symbolic pow (const Symbolic& x1, const int& x2) {
  return lift<C,V,pow> (x1, generic (x2)); }
TMPL Symbolic pow (const int& x1, const Symbolic& x2) {
  return lift<C,V,pow> (generic (x1), x2); }

TMPL Symbolic sqrt (const Symbolic& x1) {
  return lift<C,V,sqrt> (x1); }
TMPL Symbolic exp (const Symbolic& x1) {
  return lift<C,V,exp> (x1); }
TMPL Symbolic log (const Symbolic& x1) {
  return lift<C,V,log> (x1); }
TMPL Symbolic cos (const Symbolic& x1) {
  return lift<C,V,cos> (x1); }
TMPL Symbolic sin (const Symbolic& x1) {
  return lift<C,V,sin> (x1); }
TMPL Symbolic tan (const Symbolic& x1) {
  return lift<C,V,tan> (x1); }
TMPL Symbolic acos (const Symbolic& x1) {
  return lift<C,V,acos> (x1); }
TMPL Symbolic asin (const Symbolic& x1) {
  return lift<C,V,asin> (x1); }
TMPL Symbolic atan (const Symbolic& x1) {
  return lift<C,V,atan> (x1); }
TMPL Symbolic trig (const Symbolic& x1) {
  return lift<C,V,trig> (x1); }

TMPL bool operator == (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,equal_operator> (x1, x2); }
TMPL bool operator != (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,unequal_operator> (x1, x2); }
TMPL bool operator <= (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,less_operator> (x1, x2); }
TMPL bool operator < (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,lesseq_operator> (x1, x2); }
TMPL bool operator >= (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,gtr_operator> (x1, x2); }
TMPL bool operator > (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,gtreq_operator> (x1, x2); }

TMPL Symbolic derive (const Symbolic& x1) {
  return lift<C,V,derive> (x1); }
TMPL Symbolic derive (const Symbolic& x1, const Symbolic& v) {
  return lift<C,V,derive> (x1, v); }
TMPL Symbolic integrate (const Symbolic& x1) {
  return lift<C,V,integrate> (x1); }
TMPL Symbolic integrate (const Symbolic& x1, const Symbolic& v) {
  return lift<C,V,integrate> (x1, v); }
TMPL Symbolic compose (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,compose> (x1, x2); }
TMPL Symbolic dilate (const Symbolic& x1, const Symbolic& x2) {
  return lift<C,V,dilate> (x1, x2); }
TMPL Symbolic prime (const Symbolic& x1) {
  return lift<C,V,prime> (x1); }

ARITH_INT_SUGAR(TMPL,Symbolic,Symbolic);
COMPARE_INT_SUGAR(TMPL,Symbolic,Symbolic);

#undef TMPL_DEF
#undef TMPL
#undef Symbolic
} // namespace mmx
#endif // __SYMBOLIC_HPP
